@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclClock.c,v 1.16 2002/07/05 11:16:01 rmax Exp $
+ * RCS: @(#) $Id: tclClock.c,v 1.20.2.2 2005/03/15 16:29:53 kennykb Exp $
  */
 
 #include "tcl.h"
@@ -29,7 +29,7 @@ TCL_DECLARE_MUTEX(clockMutex)
  */
 
 static int		FormatClock _ANSI_ARGS_((Tcl_Interp *interp,
-			    unsigned long clockVal, int useGMT,
+			    Tcl_WideInt clockVal, int useGMT,
 			    char *format));
 
 /*
@@ -62,10 +62,11 @@ Tcl_ClockObjCmd (client, interp, objc, objv)
     int useGMT = 0;
     char *format = "%a %b %d %X %Z %Y";
     int dummy;
-    unsigned long baseClock, clockVal;
+    Tcl_WideInt baseClock, clockVal;
     long zone;
     Tcl_Obj *baseObjPtr = NULL;
     char *scanStr;
+    int n;
     
     static CONST char *switches[] =
 	{"clicks", "format", "scan", "seconds", (char *) NULL};
@@ -90,9 +91,10 @@ Tcl_ClockObjCmd (client, interp, objc, objv)
 	    int forceMilli = 0;
 
 	    if (objc == 3) {
-		format = Tcl_GetStringFromObj(objv[2], &index);
-		if (strncmp(format, "-milliseconds",
-			(unsigned int) index) == 0) {
+		format = Tcl_GetStringFromObj(objv[2], &n);
+		if ( ( n >= 2 ) 
+		     && ( strncmp( format, "-milliseconds",
+				   (unsigned int) n) == 0 ) ) {
 		    forceMilli = 1;
 		} else {
 		    Tcl_AppendStringsToObj(resultPtr,
@@ -126,7 +128,7 @@ Tcl_ClockObjCmd (client, interp, objc, objv)
 		return TCL_ERROR;
 	    }
 
-	    if (Tcl_GetLongFromObj(interp, objv[2], (long*) &clockVal)
+	    if (Tcl_GetWideIntFromObj(interp, objv[2], &clockVal)
 		    != TCL_OK) {
 		return TCL_ERROR;
 	    }
@@ -155,7 +157,7 @@ Tcl_ClockObjCmd (client, interp, objc, objv)
 	    if (objc != 0) {
 		goto wrongFmtArgs;
 	    }
-	    return FormatClock(interp, (unsigned long) clockVal, useGMT,
+	    return FormatClock(interp, clockVal, useGMT,
 		    format);
 
 	case COMMAND_SCAN:			/* scan */
@@ -192,8 +194,8 @@ Tcl_ClockObjCmd (client, interp, objc, objv)
 	    }
 
 	    if (baseObjPtr != NULL) {
-		if (Tcl_GetLongFromObj(interp, baseObjPtr,
-			(long*) &baseClock) != TCL_OK) {
+		if (Tcl_GetWideIntFromObj(interp, baseObjPtr,
+					  &baseClock) != TCL_OK) {
 		    return TCL_ERROR;
 		}
 	    } else {
@@ -203,13 +205,13 @@ Tcl_ClockObjCmd (client, interp, objc, objv)
 	    if (useGMT) {
 		zone = -50000; /* Force GMT */
 	    } else {
-		zone = TclpGetTimeZone((unsigned long) baseClock);
+		zone = TclpGetTimeZone(baseClock);
 	    }
 
 	    scanStr = Tcl_GetStringFromObj(objv[2], &dummy);
 	    Tcl_MutexLock(&clockMutex);
-	    if (TclGetDate(scanStr, (unsigned long) baseClock, zone,
-		    (unsigned long *) &clockVal) < 0) {
+	    if (TclGetDate(scanStr, baseClock, zone,
+		    &clockVal) < 0) {
 		Tcl_MutexUnlock(&clockMutex);
 		Tcl_AppendStringsToObj(resultPtr,
 			"unable to convert date-time string \"",
@@ -218,7 +220,7 @@ Tcl_ClockObjCmd (client, interp, objc, objv)
 	    }
 	    Tcl_MutexUnlock(&clockMutex);
 
-	    Tcl_SetLongObj(resultPtr, (long) clockVal);
+	    Tcl_SetWideIntObj(resultPtr, clockVal);
 	    return TCL_OK;
 
 	case COMMAND_SECONDS:			/* seconds */
@@ -253,7 +255,7 @@ Tcl_ClockObjCmd (client, interp, objc, objv)
 static int
 FormatClock(interp, clockVal, useGMT, format)
     Tcl_Interp *interp;			/* Current interpreter. */
-    unsigned long clockVal;	       	/* Time in seconds. */
+    Tcl_WideInt clockVal;	       	/* Time in seconds. */
     int useGMT;				/* Boolean */
     char *format;			/* Format string */
 {
@@ -263,7 +265,7 @@ FormatClock(interp, clockVal, useGMT, format)
     char *p;
     int result;
     time_t tclockVal;
-#ifndef HAVE_TM_ZONE
+#if !defined(HAVE_TM_ZONE) && !defined(WIN32)
     int savedTimeZone = 0;	/* lint. */
     char *savedTZEnv = NULL;	/* lint. */
 #endif
@@ -293,9 +295,12 @@ FormatClock(interp, clockVal, useGMT, format)
     /*
      * This is a kludge for systems not having the timezone string in
      * struct tm.  No matter what was specified, they use the local
-     * timezone string.
+     * timezone string.  Since this kludge requires fiddling with the
+     * TZ environment variable, it will mess up if done on multiple
+     * threads at once.  Protect it with a the clock mutex.
      */
 
+    Tcl_MutexLock( &clockMutex );
     if (useGMT) {
         CONST char *varValue;
 
@@ -305,14 +310,14 @@ FormatClock(interp, clockVal, useGMT, format)
         } else {
             savedTZEnv = NULL;
 	}
-        Tcl_SetVar2(interp, "env", "TZ", "GMT", TCL_GLOBAL_ONLY);
+        Tcl_SetVar2(interp, "env", "TZ", "GMT0", TCL_GLOBAL_ONLY);
         savedTimeZone = timezone;
         timezone = 0;
         tzset();
     }
 #endif
 
-    tclockVal = clockVal;
+    tclockVal = (time_t) clockVal;
     timeDataPtr = TclpGetDate((TclpTime_t) &tclockVal, useGMT);
     
     /*
@@ -332,10 +337,16 @@ FormatClock(interp, clockVal, useGMT, format)
     Tcl_DStringInit(&buffer);
     Tcl_DStringSetLength(&buffer, bufSize);
 
+    /* If we haven't locked the clock mutex up above, lock it now. */
+
+#if defined(HAVE_TM_ZONE) || defined(WIN32)
     Tcl_MutexLock(&clockMutex);
+#endif
     result = TclpStrftime(buffer.string, (unsigned int) bufSize,
 	    Tcl_DStringValue(&uniBuffer), timeDataPtr, useGMT);
+#if defined(HAVE_TM_ZONE) || defined(WIN32)
     Tcl_MutexUnlock(&clockMutex);
+#endif
     Tcl_DStringFree(&uniBuffer);
 
 #if !defined(HAVE_TM_ZONE) && !defined(WIN32)
@@ -349,6 +360,7 @@ FormatClock(interp, clockVal, useGMT, format)
         timezone = savedTimeZone;
         tzset();
     }
+    Tcl_MutexUnlock( &clockMutex );
 #endif
 
     if (result == 0) {
