@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclEvent.c,v 1.28.2.1 2003/05/13 12:44:07 dkf Exp $
+ * RCS: @(#) $Id: tclEvent.c,v 1.28.2.11 2005/06/24 18:21:39 kennykb Exp $
  */
 
 #include "tclInt.h"
@@ -100,8 +100,20 @@ static Tcl_ThreadDataKey dataKey;
 
 /*
  * Common string for the library path for sharing across threads.
+ * This is ckalloc'd and cleared in Tcl_Finalize.
  */
-char *tclLibraryPathStr;
+static char *tclLibraryPathStr = NULL;
+
+
+#ifdef TCL_THREADS
+
+typedef struct {
+    Tcl_ThreadCreateProc *proc;	/* Main() function of the thread */
+    ClientData clientData;	/* The one argument to Main() */
+} ThreadClientData;
+static Tcl_ThreadCreateType NewThreadProc _ANSI_ARGS_((
+           ClientData clientData));
+#endif
 
 /*
  * Prototypes for procedures referenced only in this file:
@@ -593,6 +605,8 @@ TclSetLibraryPath(pathPtr)
 				 * the new library path. */
 {
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
+    const char *toDupe;
+    int size;
 
     if (pathPtr != NULL) {
 	Tcl_IncrRefCount(pathPtr);
@@ -606,7 +620,12 @@ TclSetLibraryPath(pathPtr)
      *  No mutex locking is needed here as up the stack we're within
      *  TclpInitLock().
      */
-    tclLibraryPathStr = Tcl_GetStringFromObj(pathPtr, NULL);
+    if (tclLibraryPathStr != NULL) {
+	ckfree(tclLibraryPathStr);
+    }
+    toDupe = Tcl_GetStringFromObj(pathPtr, &size);
+    tclLibraryPathStr = ckalloc((unsigned)size+1);
+    memcpy(tclLibraryPathStr, toDupe, (unsigned)size+1);
 }
 
 /*
@@ -712,6 +731,7 @@ TclInitSubsystems(argv0)
 	     * interesting happens so we can use the allocators in the
 	     * implementation of self-initializing locks.
 	     */
+
 #if USE_TCLALLOC
 	    TclInitAlloc(); /* process wide mutex init */
 #endif
@@ -720,10 +740,10 @@ TclInitSubsystems(argv0)
 #endif
 
 	    TclpInitPlatform(); /* creates signal handler(s) */
-    	    TclInitObjSubsystem(); /* register obj types, create mutexes */
+	    TclInitObjSubsystem(); /* register obj types, create mutexes */
 	    TclInitIOSubsystem(); /* inits a tsd key (noop) */
 	    TclInitEncodingSubsystem(); /* process wide encoding init */
-    	    TclInitNamespaceSubsystem(); /* register ns obj type (mutexed) */
+	    TclInitNamespaceSubsystem(); /* register ns obj type (mutexed) */
 	}
 	TclpInitUnlock();
     }
@@ -763,6 +783,29 @@ void
 Tcl_Finalize()
 {
     ExitHandler *exitPtr;
+    
+    /*
+     * Invoke exit handlers first.
+     */
+
+    Tcl_MutexLock(&exitMutex);
+    inFinalize = 1;
+    for (exitPtr = firstExitPtr; exitPtr != NULL; exitPtr = firstExitPtr) {
+	/*
+	 * Be careful to remove the handler from the list before
+	 * invoking its callback.  This protects us against
+	 * double-freeing if the callback should call
+	 * Tcl_DeleteExitHandler on itself.
+	 */
+
+	firstExitPtr = exitPtr->nextPtr;
+	Tcl_MutexUnlock(&exitMutex);
+	(*exitPtr->proc)(exitPtr->clientData);
+	ckfree((char *) exitPtr);
+	Tcl_MutexLock(&exitMutex);
+    }    
+    firstExitPtr = NULL;
+    Tcl_MutexUnlock(&exitMutex);
 
     TclpInitLock();
     if (subsystemsInitialized != 0) {
@@ -775,29 +818,6 @@ Tcl_Finalize()
 
 	(void) TCL_TSD_INIT(&dataKey);
 
-	/*
-	 * Invoke exit handlers first.
-	 */
-
-	Tcl_MutexLock(&exitMutex);
-	inFinalize = 1;
-	for (exitPtr = firstExitPtr; exitPtr != NULL; exitPtr = firstExitPtr) {
-	    /*
-	     * Be careful to remove the handler from the list before
-	     * invoking its callback.  This protects us against
-	     * double-freeing if the callback should call
-	     * Tcl_DeleteExitHandler on itself.
-	     */
-
-	    firstExitPtr = exitPtr->nextPtr;
-	    Tcl_MutexUnlock(&exitMutex);
-	    (*exitPtr->proc)(exitPtr->clientData);
-	    ckfree((char *) exitPtr);
-	    Tcl_MutexLock(&exitMutex);
-	}    
-	firstExitPtr = NULL;
-	Tcl_MutexUnlock(&exitMutex);
-
 	/*
 	 * Clean up after the current thread now, after exit handlers.
 	 * In particular, the testexithandler command sets up something
@@ -842,6 +862,10 @@ Tcl_Finalize()
 	    ckfree(tclDefaultEncodingDir);
 	    tclDefaultEncodingDir = NULL;
 	}
+	if (tclLibraryPathStr != NULL) {
+	    ckfree(tclLibraryPathStr);
+	    tclLibraryPathStr = NULL;
+	}
 	
 	Tcl_SetPanicProc(NULL);
 
@@ -870,14 +894,34 @@ Tcl_Finalize()
 	TclFinalizeLoad();
 	TclResetFilesystem();
 	
+	/*
+	 * There have been several bugs in the past that cause
+	 * exit handlers to be established during Tcl_Finalize
+	 * processing.  Such exit handlers leave malloc'ed memory,
+	 * and Tcl_FinalizeThreadAlloc or Tcl_FinalizeMemorySubsystem
+	 * will result in a corrupted heap.  The result can be a
+	 * mysterious crash on process exit.  Check here that
+	 * nobody's done this.
+	 */
+
+#ifdef TCL_MEM_DEBUG
+	if ( firstExitPtr != NULL ) {
+	    Tcl_Panic( "exit handlers were created during Tcl_Finalize" );
+	}
+#endif
+
 	/*
 	 * There shouldn't be any malloc'ed memory after this.
 	 */
+	TclFinalizePreserve();
+#if defined(TCL_THREADS) && defined(USE_THREAD_ALLOC) && !defined(TCL_MEM_DEBUG) && !defined(PURIFY)
+	TclFinalizeThreadAlloc();
+#endif
 
 	TclFinalizeMemorySubsystem();
 	inFinalize = 0;
     }
-    TclpInitUnlock();
+    TclFinalizeLock();
 }
 
 /*
@@ -1136,3 +1180,79 @@ Tcl_UpdateObjCmd(clientData, interp, objc, objv)
     Tcl_ResetResult(interp);
     return TCL_OK;
 }
+
+#ifdef TCL_THREADS
+/*
+ *-----------------------------------------------------------------------------
+ *
+ *  NewThreadProc --
+ *
+ * 	Bootstrap function of a new Tcl thread.
+ *
+ * Results:
+ *	None.
+ *
+ * Side Effects:
+ *	Initializes Tcl notifier for the current thread.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static Tcl_ThreadCreateType
+NewThreadProc(ClientData clientData)
+{
+    ThreadClientData *cdPtr;
+    ClientData threadClientData;
+    Tcl_ThreadCreateProc *threadProc;
+
+    cdPtr = (ThreadClientData*)clientData;
+    threadProc = cdPtr->proc;
+    threadClientData = cdPtr->clientData;
+    Tcl_Free((char*)clientData); /* Allocated in Tcl_CreateThread() */
+
+    (*threadProc)(threadClientData);
+
+    TCL_THREAD_CREATE_RETURN;
+}
+#endif
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tcl_CreateThread --
+ *
+ *	This procedure creates a new thread. This actually belongs
+ *	to the tclThread.c file but since we use some private 
+ *	data structures local to this file, it is placed here.
+ *
+ * Results:
+ *	TCL_OK if the thread could be created.  The thread ID is
+ *	returned in a parameter.
+ *
+ * Side effects:
+ *	A new thread is created.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+Tcl_CreateThread(idPtr, proc, clientData, stackSize, flags)
+    Tcl_ThreadId *idPtr;		/* Return, the ID of the thread */
+    Tcl_ThreadCreateProc proc;		/* Main() function of the thread */
+    ClientData clientData;		/* The one argument to Main() */
+    int stackSize;			/* Size of stack for the new thread */
+    int flags;				/* Flags controlling behaviour of
+					 * the new thread */
+{
+#ifdef TCL_THREADS
+    ThreadClientData *cdPtr;
+
+    cdPtr = (ThreadClientData*)Tcl_Alloc(sizeof(ThreadClientData));
+    cdPtr->proc = proc;
+    cdPtr->clientData = clientData;
+
+    return TclpThreadCreate(idPtr, NewThreadProc, (ClientData)cdPtr,
+                           stackSize, flags);
+#else
+    return TCL_ERROR;
+#endif /* TCL_THREADS */
+}
