@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclUnixFCmd.c,v 1.28.2.2 2003/10/03 17:45:37 vincentdarley Exp $
+ * RCS: @(#) $Id: tclUnixFCmd.c,v 1.28.2.4 2005/01/10 11:21:51 dkf Exp $
  *
  * Portions of this code were derived from NetBSD source code which has
  * the following copyright notice:
@@ -147,7 +147,7 @@ static int		TraversalDelete _ANSI_ARGS_((Tcl_DString *srcPtr,
 static int		TraverseUnixTree _ANSI_ARGS_((
 			    TraversalProc *traversalProc,
 			    Tcl_DString *sourcePtr, Tcl_DString *destPtr,
-			    Tcl_DString *errorPtr));
+			    Tcl_DString *errorPtr, int doRewind));
 
 #ifdef PURIFY
 /*
@@ -433,8 +433,8 @@ CopyFile(src, dst, statBufPtr)
 {
     int srcFd;
     int dstFd;
-    u_int blockSize;   /* Optimal I/O blocksize for filesystem */
-    char *buffer;      /* Data buffer for copy */
+    unsigned blockSize;		/* Optimal I/O blocksize for filesystem */
+    char *buffer;		/* Data buffer for copy */
     size_t nread;
 
     if ((srcFd = TclOSopen(src, O_RDONLY, 0)) < 0) {	/* INTL: Native. */
@@ -641,7 +641,7 @@ TclpObjCopyDirectory(srcPathPtr, destPathPtr, errorPtr)
 	Tcl_DecrRefCount(transPtr);
     }
 
-    ret = TraverseUnixTree(TraversalCopy, &srcString, &dstString, &ds);
+    ret = TraverseUnixTree(TraversalCopy, &srcString, &dstString, &ds, 0);
 
     Tcl_DStringFree(&srcString);
     Tcl_DStringFree(&dstString);
@@ -760,7 +760,7 @@ DoRemoveDirectory(pathPtr, recursive, errorPtr)
      */
 
     if (result == TCL_OK) {
-	result = TraverseUnixTree(TraversalDelete, pathPtr, NULL, errorPtr);
+	result = TraverseUnixTree(TraversalDelete, pathPtr, NULL, errorPtr, 1);
     }
     
     if ((result != TCL_OK) && (recursive != 0)) {
@@ -793,7 +793,7 @@ DoRemoveDirectory(pathPtr, recursive, errorPtr)
  */
 
 static int 
-TraverseUnixTree(traverseProc, sourcePtr, targetPtr, errorPtr)
+TraverseUnixTree(traverseProc, sourcePtr, targetPtr, errorPtr, doRewind)
     TraversalProc *traverseProc;/* Function to call for every file and
 				 * directory in source hierarchy. */
     Tcl_DString *sourcePtr;	/* Pathname of source directory to be
@@ -803,11 +803,18 @@ TraverseUnixTree(traverseProc, sourcePtr, targetPtr, errorPtr)
     Tcl_DString *errorPtr;	/* If non-NULL, uninitialized or free
 				 * DString filled with UTF-8 name of file
 				 * causing error. */
+    int doRewind;		/* Flag indicating that to ensure complete
+    				 * traversal of source hierarchy, the readdir
+    				 * loop should be rewound whenever
+    				 * traverseProc has returned TCL_OK; this is
+    				 * required when traverseProc modifies the
+    				 * source hierarchy, e.g. by deleting files. */ 
 {
     Tcl_StatBuf statBuf;
     CONST char *source, *errfile;
     int result, sourceLen;
     int targetLen;
+    int needRewind;
     Tcl_DirEntry *dirEntPtr;
     DIR *dirPtr;
 
@@ -852,36 +859,45 @@ TraverseUnixTree(traverseProc, sourcePtr, targetPtr, errorPtr)
 	targetLen = Tcl_DStringLength(targetPtr);
     }
 
-    while ((dirEntPtr = TclOSreaddir(dirPtr)) != NULL) { /* INTL: Native. */
-	if ((dirEntPtr->d_name[0] == '.')
-		&& ((dirEntPtr->d_name[1] == '\0')
-			|| (strcmp(dirEntPtr->d_name, "..") == 0))) {
-	    continue;
+    do {
+	needRewind = 0;
+	while ((dirEntPtr = TclOSreaddir(dirPtr)) != NULL) { /* INTL: Native. */
+	    if ((dirEntPtr->d_name[0] == '.')
+		    && ((dirEntPtr->d_name[1] == '\0')
+			    || (strcmp(dirEntPtr->d_name, "..") == 0))) {
+		continue;
+	    }
+	    
+	    /* 
+	     * Append name after slash, and recurse on the file.
+	     */
+	    
+	    Tcl_DStringAppend(sourcePtr, dirEntPtr->d_name, -1);
+	    if (targetPtr != NULL) {
+		Tcl_DStringAppend(targetPtr, dirEntPtr->d_name, -1);
+	    }
+	    result = TraverseUnixTree(traverseProc, sourcePtr, targetPtr,
+		    errorPtr, doRewind);
+	    if (result != TCL_OK) {
+	    	needRewind = 0;
+		break;
+	    } else {
+		needRewind = doRewind;
+	    }
+	    
+	    /*
+	     * Remove name after slash.
+	     */
+	    
+	    Tcl_DStringSetLength(sourcePtr, sourceLen);
+	    if (targetPtr != NULL) {
+		Tcl_DStringSetLength(targetPtr, targetLen);
+	    }
 	}
-
-	/* 
-	 * Append name after slash, and recurse on the file.
-	 */
-
-	Tcl_DStringAppend(sourcePtr, dirEntPtr->d_name, -1);
-	if (targetPtr != NULL) {
-	    Tcl_DStringAppend(targetPtr, dirEntPtr->d_name, -1);
+	if (needRewind) {
+	    rewinddir(dirPtr);
 	}
-	result = TraverseUnixTree(traverseProc, sourcePtr, targetPtr,
-		errorPtr);
-	if (result != TCL_OK) {
-	    break;
-	}
-	
-	/*
-	 * Remove name after slash.
-	 */
-
-	Tcl_DStringSetLength(sourcePtr, sourceLen);
-	if (targetPtr != NULL) {
-	    Tcl_DStringSetLength(targetPtr, targetLen);
-	}
-    }
+    } while (needRewind);
     closedir(dirPtr);
     
     /*
