@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclWinThrd.c,v 1.24.2.8 2004/07/21 01:30:58 hobbs Exp $
+ * RCS: @(#) $Id: tclWinThrd.c,v 1.24.2.11 2005/05/30 01:36:53 hobbs Exp $
  */
 
 #include "tclWinInt.h"
@@ -87,6 +87,20 @@ static Tcl_ThreadDataKey dataKey;
 
 #endif /* TCL_THREADS */
 
+/*
+ * Additions by AOL for specialized thread memory allocator.
+ */
+
+#if defined(USE_THREAD_ALLOC) && !defined(TCL_MEM_DEBUG)
+static int   once;
+static DWORD tlsKey;
+
+typedef struct allocMutex {
+    Tcl_Mutex        tlock;
+    CRITICAL_SECTION wlock;
+} allocMutex;
+#endif
+
 /*
  * State bits for the thread.
  * WIN_THREAD_UNINIT		Uninitialized.  Must be zero because
@@ -680,13 +694,16 @@ TclpFinalizeThreadData(keyPtr)
     DWORD *indexPtr;
     BOOL success;
 
-#ifdef USE_THREAD_ALLOC
-    TclWinFreeAllocCache();
-#endif
     if (*keyPtr != NULL) {
 	indexPtr = *(DWORD **)keyPtr;
 	result = (VOID *)TlsGetValue(*indexPtr);
 	if (result != NULL) {
+#if defined(USE_THREAD_ALLOC) && !defined(TCL_MEM_DEBUG)
+	    if (indexPtr == &tlsKey) {
+		TclpFreeAllocCache(result);
+		return;
+	    }
+#endif
 	    ckfree((char *)result);
 	    success = TlsSetValue(*indexPtr, (void *)NULL);
             if (!success) {
@@ -1037,15 +1054,8 @@ TclpFinalizeCondition(condPtr)
 /*
  * Additions by AOL for specialized thread memory allocator.
  */
-#ifdef USE_THREAD_ALLOC
-static int once;
-static DWORD key;
-
-typedef struct allocMutex {
-    Tcl_Mutex        tlock;
-    CRITICAL_SECTION wlock;
-} allocMutex;
 
+#if defined(USE_THREAD_ALLOC) && !defined(TCL_MEM_DEBUG)
 Tcl_Mutex *
 TclpNewAllocMutex(void)
 {
@@ -1077,18 +1087,18 @@ TclpGetAllocCache(void)
 
     if (!once) {
 	/*
-	 * We need to make sure that TclWinFreeAllocCache is called
+	 * We need to make sure that TclpFreeAllocCache is called
 	 * on each thread that calls this, but only on threads that
 	 * call this.
 	 */
-    	key = TlsAlloc();
+    	tlsKey = TlsAlloc();
 	once = 1;
-	if (key == TLS_OUT_OF_INDEXES) {
+	if (tlsKey == TLS_OUT_OF_INDEXES) {
 	    panic("could not allocate thread local storage");
 	}
     }
 
-    result = TlsGetValue(key);
+    result = TlsGetValue(tlsKey);
     if ((result == NULL) && (GetLastError() != NO_ERROR)) {
         panic("TlsGetValue failed from TclpGetAllocCache!");
     }
@@ -1099,37 +1109,35 @@ void
 TclpSetAllocCache(void *ptr)
 {
     BOOL success;
-    success = TlsSetValue(key, ptr);
+    success = TlsSetValue(tlsKey, ptr);
     if (!success) {
         panic("TlsSetValue failed from TclpSetAllocCache!");
     }
 }
 
 void
-TclWinFreeAllocCache(void)
+TclpFreeAllocCache(void *ptr)
 {
-    void *ptr;
     BOOL success;
 
-    ptr = TlsGetValue(key);
     if (ptr != NULL) {
-	success = TlsSetValue(key, NULL);
+        /*
+         * Called by the pthread lib when a thread exits
+         */
+        TclFreeAllocCache(ptr);
+        success = TlsSetValue(tlsKey, NULL);
         if (!success) {
-            panic("TlsSetValue failed from TclWinFreeAllocCache!");
+            panic("TlsSetValue failed from TclpFreeAllocCache!");
         }
-	TclFreeAllocCache(ptr);
-    } else {
-      if (GetLastError() != NO_ERROR) {
-          panic("TlsGetValue failed from TclWinFreeAllocCache!");
-      }
-    }
-
-    if (once) {    
-        success = TlsFree(key);
+    } else if (once) { 
+        /*
+         * Called by us in TclFinalizeThreadAlloc() during
+         * the library finalization initiated from Tcl_Finalize()
+         */
+        success = TlsFree(tlsKey);
         if (!success) {
-            Tcl_Panic("TlsFree failed from TclWinFreeAllocCache!");
+            Tcl_Panic("TlsFree failed from TclpFreeAllocCache!");
         }
-
         once = 0; /* reset for next time. */
     }
 }
